﻿using System;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Collections.Generic;

namespace MSHO.Collection.Service.IO
{
    /// <summary>
    /// IOCP服务主程序
    /// </summary>
    public partial class IOServer : IDisposable
    {
        private bool m_Running = false;
        private bool m_Disposed = false;
        private int m_MaxConnections;
        private int m_BufferSize;
        private string m_IPAddress;
        private int m_Port;
        private System.Net.Sockets.Socket m_ListenSocket;
        private System.Threading.Tasks.Task m_ServerThread;

        /// <summary>
        /// IOCP服务主程序
        /// </summary>
        /// <param name="ip">IP地址</param>
        /// <param name="port">端口</param>
        /// <param name="maxConnections">最大连接数</param>
        /// <param name="bufferSize">缓冲区大小</param>
        public IOServer(string ip, int port, int maxConnections, int bufferSize)
        {
            #region check arguments
            if (maxConnections <= 0)
            {
                throw new ArgumentException("invalid maxConnections");
            }

            if (bufferSize <= 0)
            {
                throw new ArgumentException("invalid bufferSize");
            }

            if (port <= 0)
            {
                throw new ArgumentException("invalid port");
            }
            #endregion

            m_MaxConnections = maxConnections;
            m_BufferSize = bufferSize;
            m_IPAddress = ip;
            m_Port = port;
        }

        /// <summary>
        /// 启动服务
        /// </summary>
        public void Start()
        {
            m_ServerThread = new System.Threading.Tasks.Task(AcceptClient);
            m_ServerThread.Start();
        }

        private void AcceptClient() {
            m_Running = true;

            var ipAddress = System.Net.IPAddress.Any;
            if (!System.Net.IPAddress.TryParse(m_IPAddress, out ipAddress))
            {
                ipAddress = System.Net.IPAddress.Any;
            }

            m_ListenSocket = new System.Net.Sockets.Socket(ipAddress.AddressFamily, SocketType.Stream, ProtocolType.Tcp);
            m_ListenSocket.ReceiveBufferSize = m_BufferSize;
            m_ListenSocket.SendBufferSize = m_BufferSize;
            m_ListenSocket.ReceiveTimeout = 60 * 1000;
            m_ListenSocket.SendTimeout = 30 * 1000;


            if (ipAddress.AddressFamily == AddressFamily.InterNetworkV6)
            {
                m_ListenSocket.SetSocketOption(SocketOptionLevel.IPv6, SocketOptionName.IPv6Only, false);
                m_ListenSocket.Bind(new IPEndPoint(IPAddress.IPv6Any, m_Port));
            }
            else
            {
                m_ListenSocket.Bind(new IPEndPoint(IPAddress.Any, m_Port));
            }

            m_ListenSocket.Listen(m_MaxConnections);

            try
            {
                while (m_Running)
                {
                    var client = m_ListenSocket.Accept();
                    if (client == null)
                    {
                        break;
                    }
                    else
                    {
#if DEBUG
                        Console.WriteLine("...........................................采集端连接");
#endif
                        client.ReceiveBufferSize = m_BufferSize;
                        client.SendBufferSize = m_BufferSize;
                        client.ReceiveTimeout = 60 * 1000;
                        client.SendTimeout = 30 * 1000;
                        StartClientTask(client);
                    }
                }
            }
            catch(Exception ex) {
                MSHO.Collection.DataAccess.MSHOService.Instance.SaveSystemLog("MSHO.Collection.Service", "AcceptClient", ex);
                Console.WriteLine(ex.Message);
            }
        }


        #region Close
        /// <summary>
        /// 停止服务
        /// </summary>
        public void Stop()
        {
            if (!m_Disposed)
            {
                m_Running = false;
                if (m_ListenSocket != null)
                {
                    try
                    {
                        m_ListenSocket.Shutdown(SocketShutdown.Both);
                    }
                    catch { }

                    try
                    {
                        m_ListenSocket.Close();
                    }
                    catch { }
                    m_ListenSocket = null;
                }

                if (m_ServerThread != null) {
                    m_ServerThread = null;
                }
            }
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        private void Dispose(bool disposing)
        {
            if (!m_Disposed)
            {
                this.Stop();
                m_Disposed = true;
            }
        }

        ~IOServer()
        {
            Dispose(false);
        }
        #endregion

    }

}
